#title = "＜実行禁止＞起動オプション拡張"
//==============================================================================
// コマンドライン引数の定義
// 引数に対し順に実行される
//
// コマンドライン引数の分解について
// ":" で分割した配列で渡される
// 例：/find:i:"Search" => opt[0]='/find', opt[1]='i', opt[2]='"Search"'
//
// 【戻り値】
//   true:  引数キューから削除する
//   false: 引数キューに残す
//          処理対象外の引数の場合や，設定以降全てのファイルに有効な引数の場合
//==============================================================================

//------------------------------------------------------------------------------
// 逐次実行オプション
// 【引数】
// opt: コマンドライン引数
//------------------------------------------------------------------------------
var seqOptFunc = [];

// 標準オプションを無視
seqOptFunc[seqOptFunc.length] = function(opt) {
	// 桁指定
	if (opt[0].equalsIgnoreCase("/cl")) { return 2; }		// 続く追加オプションを無視
	// 書き換え禁止
	if (opt[0].equalsIgnoreCase("/r")) { return true; }
	// ファイルから検索ダイアログ表示
	if (opt[0].equalsIgnoreCase("/fd")) { return true; }
	// マクロ実行
	if (opt[0].equalsIgnoreCase("/mf")) { return 2; }		// 続く追加オプションを無視
	// 行指定
	if (opt[0].equalsIgnoreCase("/l")) { return true; }
	// トレイアイコン表示
	if (opt[0].equalsIgnoreCase("/ti")) { return true; }

	return false;
};

// マクロを実行(/mfs)
// 標準オプションと違い「登録不要」(※セキュリティリスクは各自管理)
// 【オプション】
//   s: ファイルを Shift-JIS でロードする(デフォルトは UTF-8)
//   =: 要素名=値 で実行マクロに引数を渡せる
//      マクロからは CmdArguments でアクセス
//      例：コマンド : /mfs:id=12345:"hoje.js"
//          javscript: CmdArguments.id
seqOptFunc[seqOptFunc.length] = function(opt) {
	// 自分のオプションかを判定
	if (!opt[0].equalsIgnoreCase("/mfs")) { return false; }
	// 実行マクロファイル名がなければ処理しない
	if (opt.length <= 1 || !opt[opt.length-1].isEnquote()) { return true; }

	// オプション取得
	var charset = "utf-8";
	var macroArgStr = "";
	for (var i=1; i<opt.length-1; i++) {
		if (opt[i].indexOf("=") >= 0) {
			macroArgStr += opt[i].replace("=", ":") + ",";
		} else {
			if (opt[i].indexOf("s") >= 0) {
				charset = "shift-jis";
			}
		}
	}

	// 実行マクロ名を検索
	// 絶対パス：そのファイルがなければエラー
	// 相対パス：カレントディレクトリ ⇒ My Macros の順で検索
	var filePath = opt[opt.length-1].dequote();
	if (fso.FileExists(fso.GetAbsolutePathName(filePath))) {
		filePath = fso.GetAbsolutePathName(filePath);
	} else if (filePath.indexOf(":") < 0) {
		filePath = fso.GetParentFolderName(Editor.FullName) + "\\My Macros\\" + filePath;
		if (!fso.FileExists(filePath)) {
			filePath = null;
		}
	} else {
		filePath = null;
	}

	// マクロをロードして実行
	if (filePath) {
		var macroCode = LoadFromFile(filePath, charset);

		// マクロ内からプリプロセッサ(Mery拡張)を除外
		while (macroCode) {
			var space = macroCode.search(/\S/m);
			if (macroCode.charAt(RegExp.index) == "#") {
				var index = macroCode.indexOf("\n");
				if (index >= 0) {
					macroCode = macroCode.substring(index+1, macroCode.length);
				} else {
					macroCode = "";
				}
			} else {
				break;
			}
		}

		// 例外処理で Quit() を代替
		// ただし，可能な限り全体を (function(){})() で囲み，Quit() の代わりに return を使用する
		var QuitError = function(){ this.message = "この Error は Quit() の代わりに使われています．QuitError を処理しないでください．"; };
		QuitError.prototype = new Error();
		QuitError.prototype.name = "QuitError";
		var quitRegist = "if (!quit) { var quit = function(){ throw new QuitError(); } };\n" +
						 "if (!Quit) { var Quit = function(){ throw new QuitError(); } };\n" +
						 "if (!QUIT) { var QUIT = function(){ throw new QuitError(); } };\n";

		// マクロ内で宣言なしで変数使われるとデータ破壊される可能性があり
		// Mery 標準オブジェクト及び CmdArguments 以外は必ず var 宣言！
		(function(code, arg){
			// マクロ用引数を設定
			// 面倒なので eval で取り込み
			// セキュリティは気にしない方向で(必要あればきちんと実装しよう)
			var CmdArguments = null;
			if (arg) {
				eval("CmdArguments = {" + arg.substring(0, arg.length-1) + "};");
			}
			var ScriptName = fso.GetBaseName(filePath);
			var ScriptFullName = fso.GetFile(filePath).Path;
			
			// マクロ実行
			try {
				eval(quitRegist + code);
			} catch (e) {
				if (e instanceof QuitError) {
					// none
				} else {
					throw e;
				}
			}
		})(macroCode, macroArgStr);
	} else {
		Alert("起動オプション /mfs のエラー：\n" + opt[opt.length-1] + " が見つかりません．");
	}

	return true;
};

// マクロを実行(/mfd)
// ファイルではなく指定文字列をマクロとして実行する．
// コード中のダブルクォーテーションは重ねることでエスケープ(VB方式)
//   例： /mfd:"Alert(""hoge"");"
seqOptFunc[seqOptFunc.length] = function(opt) {
	// 自分のオプションかを判定
	if (!opt[0].equalsIgnoreCase("/mfd")) { return false; }
	// マクロ文字列がなければ終了
	if (opt.length <= 1 || !opt[opt.length-1].isEnquote()) { return true; }

	// オプション取得(使うことはないだろうけど)
	var macroArgStr = "";
	for (var i=1; i<opt.length-1; i++) {
		if (opt[i].indexOf("=") >= 0) {
			macroArgStr += opt[i].replace("=", ":") + ",";
		}
	}

	// エスケープ解除
	var code = opt[opt.length-1].dequote();

	(function(code, arg){
		var CmdArguments = null;
		if (arg) {
			eval("CmdArguments = {" + arg.substring(0, arg.length-1) + "};");
		}
		// マクロ実行
		eval(code);
	})(code, macroArgStr);
};

// ファイル更新チェック(/reload)
// ファイルが更新されている場合は自動で読み込み
// 【更新判定条件】
//  1. ドキュメントは保存済み
//  2. ファイルと内容が異なる
seqOptFunc[seqOptFunc.length] = function(opt) {
	// 自分のオプションかを判定
	if (!opt[0].equalsIgnoreCase("/reload")) { return false; }
	// 実行マクロファイル名がなければ処理しない
	if (opt.length <= 1 || !opt[opt.length-1].isEnquote()) { return true; }

	// 開いているドキュメントから一致するものを検索
	// 開いていない場合は開いて終了
	var path = opt[opt.length-1].dequote();
	var doc = SearchDocument(path);
	if (!doc) {
		Editor.NewFile();
		Editor.OpenFile(path);
		return true;
	}

	// 保存済みの場合のみ処理
	if (!doc.Saved) { return true; }

	// 内容をロードして比較
	// 異なる場合は書き換え
	var text = LoadFromFile(doc.FullName);
	if (text != null) {
		text = text.replace(/\r\n/g, "\n").replace(/\r/g, "\n");
		if (text != doc.Text) {
			var sel = doc.Selection;
			var x = sel.GetActivePointX(mePosLogical);
			var y = sel.GetActivePointY(mePosLogical);
			doc.Text = text;
			doc.Save();		// ダイアログを出さないための苦肉策
			sel.SetActivePoint(mePosLogical, x, y);
		}
	}

	return true;
};

//// カレントディレクトリ設定(/cd)
//// 指定したディレクトリをカレントに設定
//// 指定省略時は Mery の実行ファイルのディレクトリ
//seqOptFunc[seqOptFunc.length] = function(opt) {
//	// 自分のオプションかを判定
//	if (!opt[0].equalsIgnoreCase("/cd")) { return false; }

//	if (opt.length == 1) {
//		shell.CurrentDirectory = fso.GetParentFolderName(Editor.FullName);
//	} else {
//		shell.CurrentDirectory = opt[1].dequote();
//	}

//	return true;
//}

//// ファイルの自動作成(/create)
//// 存在しないファイルを開こうとした時は自動で作成
//seqOptFunc[seqOptFunc.length] = function(opt) {
//	// 自分のオプションかを判定
//	if (!opt[0].equalsIgnoreCase("/create")) { return false; }

//	$isAutoCreateNewFile = true;

//	return true;
//}

//------------------------------------------------------------------------------
// ドキュメント指定時実行オプション
// 【引数】
//   opt: コマンドライン引数
//   doc: 対象のドキュメントが開かれている場合，そのドキュメント
//------------------------------------------------------------------------------
var docOptFunc = [];

// 文字列検索(/find)
// 指定した文字列を検索する(マクロの find を利用)
// 【オプション】
//   i: 大文字小文字を区別する
//   x: 正規表現で検索
docOptFunc[docOptFunc.length] = function(opt, doc) {
	// 自分のオプションかを判定
	if (!opt[0].equalsIgnoreCase("/find")) { return false; }
	// ドキュメントがなければ処理しない
	if (!doc) { return true; }
	// 探索文字列指定がなければ処理しない
	if (opt.length <= 1 || !opt[opt.length-1].isEnquote()) { return true; }

	// 検索オプションを取得
	var flags = meFindReplaceOnlyWord;
	for (var i=1; i<opt.length-1; i++) {
		if (opt[i].indexOf("i") >= 0) {
			flags |= meFindReplaceCase;
		}
		if (opt[i].indexOf("x") >= 0) {
			flags |= meFindReplaceRegExp;
		}
	}

	// 検索処理
	var str = opt[opt.length-1].dequote();
	var sel = doc.Selection;
	sel.Find(str, flags);

	// 処理したのでオプション削除
	return true;
};

// 範囲選択(/select)
// 指定範囲を選択
// 【オプション】
//   選択範囲: "," 区切り，論理行で開始位置と終了位置
docOptFunc[docOptFunc.length] = function(opt, doc) {
	// 自分のオプションかを判定
	if (!opt[0].equalsIgnoreCase("/select")) { return false; }
	// ドキュメントがなければ処理しない
	if (!doc) { return true; }

	// 範囲取得
	if (opt.length <= 1) { return true; }
	var a = opt[opt.length-1].split(",");
	if (a.length < 4) { return true; }

	// 範囲選択
	var sel = doc.Selection;
	sel.SetActivePoint(mePosLogical, Number(a[0]), Number(a[1]));
	sel.SetAnchorPoint(mePosLogical, Number(a[2]), Number(a[3]));

	return true;
};


//------------------------------------------------------------------------------
// 完了時実行オプション
// 【引数】
//   opt: コマンドライン引数
//   docs: 開いたドキュメント一覧
//------------------------------------------------------------------------------
var lastOptFunc = [];

// プロジェクト管理(/prj)
// 拡張子 .mprj (Mery Project) を開いたとき，プロジェクトファイルとして処理する
// といっても羅列されたフルパスを全て開くだけ
lastOptFunc[lastOptFunc.length] = function(opt, docs) {
	// 自分のオプションかを判定
	if (!opt[0].equalsIgnoreCase("/prj")) { return false; }

	// 拡張子が .mprj のファイルを探し，一覧を開いて該当の doc は閉じる
	for (var i=0; i<docs.length; i++) {
		var doc = docs[i];
		if (doc.Name.endsWith(".mprj", true)) {
			var a = doc.Text.split("\n");
			for (var j=0; j<a.length; j++) {
				if (!a[j] || a[j].startsWith("#")) {
					continue;
				}
				Editor.NewFile();
				Editor.OpenFile(a[j]);
			}

			// ドキュメントを閉じ，対象一覧から削除
			doc.Close();
			docs.splice(i, 1);
			i--;
		}
	}

	// テキストが未入力の『無題』を閉じる(ゴミ処理)
	for (var i=0; i<Editor.Documents.Count && Editor.Documents.Count > 1; i++) {
		var doc = Editor.Documents.Item(i);
		if (!doc.FullName && !doc.Text) {
			doc.Close();
			i--;
		}
	}
};

//==============================================================================
// ユーティリティ関数定義
//==============================================================================
// エンクォート判定
String.prototype.isEnquote = function() {
	return this.length >= 2 && this.charAt(0) == '"' && this.charAt(this.length-1) == '"';
};
// デクォート処理
String.prototype.dequote = function() {
	if (this.isEnquote()) {
		return this.substring(1, this.length-1).replace(/""/g, '"');
	}
	return this;
};
// エンクォート処理
String.prototype.enquote = function() {
	return '"' + this.replace(/"/g, '""') + '"';
};
// 大文字小文字を区別しない文字列一致判定
String.prototype.equalsIgnoreCase = function(str) {
	return this.toUpperCase() == str.toUpperCase();
};
// 開始文字列一致判定
String.prototype.startsWith = function(str, isIgnoreCase) {
	if (!isIgnoreCase) {
		return this.length >= str.length && this.substring(0, str.length) == str;
	} else {
		return this.length >= str.length && str.equalsIgnoreCase(this.substring(0, str.length));
	}
};
// 最後尾文字列一致判定
String.prototype.endsWith = function(str, isIgnoreCase) {
	if (!isIgnoreCase) {
		return this.length >= str.length && this.substring(this.length - str.length, this.length) == str;
	} else {
		return this.length >= str.length && str.equalsIgnoreCase(this.substring(this.length - str.length, this.length));
	}
};

//// 出現回数カウント
//String.prototype.count = function(s, b, e) {
//	b = b === undefine ? 0 : b;
//	e = e === undefine ? this.length : e;
//	var count = 0;
//	var length = s.length;
//	var index = this.indexOf(s, b);
//	while (index >= 0 && index < e) {
//		count++;
//		index = this.indexOf(s, index+length);
//	}
//	return count;
//};

// エンクォート含む文字列の分割処理
var SplitWithQuote = function(str, delimiter) {
	var a = [];
	var unDel = new RegExp("[^"+ delimiter + "]");

	while (str) {
		// 連続するデリミタを無視
		var b = str.search(unDel);
		if (b < 0) {
			break;
		}

		// ダブルクォーテーション外のデリミタを検索
		var space = str.indexOf(delimiter, b);
		var quote = str.indexOf('"', b);
		while (quote >= 0 && quote < space) {
			quote = str.indexOf('"', quote+1);
			if (quote >= 0) {
				space = str.indexOf(delimiter, quote+1);
				quote = str.indexOf('"', quote+1);
			}
		}

		// デリミタが見つからない場合は残り全てをオプションとして取得
		if (space < 0) {
			space = str.length;
		}

		// オプションを取得して解析済み文字列を削除
		a[a.length] = str.substring(b, space);
		str = str.substring(space);
	}

	return a;
};

// ファイル読み込み
// 存在しない場合は null を返す
function LoadFromFile(path, charset) {
	if (!fso.FileExists(path)) {
		return null;
	}
	if (!charset) {
		charset = "utf-8";
	}
	var adodb = new ActiveXObject("ADODB.Stream");
	adodb.Charset = charset;
	adodb.Type = 2;
	adodb.Open();
	adodb.LoadFromFile(path);
	var text = adodb.ReadText(-1);
	adodb.Close();
	return text;
};

// ファイル保存
function SaveToFile(path, text, charset, hasBOM) {
	if (!charset) {
		charset = "utf-8";
	}
	var adodb = new ActiveXObject("ADODB.Stream");
	adodb.Charset = charset;
	adodb.Type = 2;
	adodb.Open();
	adodb.WriteText(text);

	// BOM 削除処理
	if (!hasBOM && charset.equalsIgnoreCase("utf-8")) {
		adodb.Position = 0;
		adodb.Type = 1;
		adodb.Position = 3;
		var binary = adodb.Read();
		adodb.Close();

		// 新規ストリームに BOM を除いた文字列を書き込む
		adodb.Open();
		adodb.Write(binary);
	}
	adodb.SaveToFile(path, 2);
	adodb.Close();
};

// 正規化パスとドキュメントの対応表を作成
// パス：パス区切りは \
//       文字は全て大文字
var documentTable = {};
function UpdateDocumentTable() {
	documentTable = {};
	for (var i=0; i<Editors.Count; i++) {
		var edit = Editors.Item(i);
			for (var j=0; j<edit.documents.Count; j++) {
			var doc = edit.documents.Item(j);
			if (doc.FullName) {
				documentTable[doc.FullName.replace(/\//g, "\\").toUpperCase()] = doc;
			}
		}
	}
};
UpdateDocumentTable();

// パスから Mery で開いているドキュメントオブジェクトを取得
function SearchDocument(path) {
	path = fso.GetAbsolutePathName(path).replace(/\//g, "\\").toUpperCase();
	var doc = documentTable[path]
	return doc ? doc : null;
};


//==============================================================================
// 処理開始
//==============================================================================
// コマンドライン文字列取得
var shell = new ActiveXObject("WScript.Shell");
var exec = shell.Exec('WScript.exe "' + ScriptFullName.match(/.*\\/)+'\\[WSH] GetCmd.js" "' + Editor.FullName + '"');
while (exec.Status == 0) {
	Sleep(10);
}
cmd = exec.StdOut.ReadAll();

// ルール
// ① 1 つ目は実行ファイル名
// ② / で始まる塊はオプション
// ③ / で始まらない文字列はファイル名
// ④ 標準オプションは処理しない
var args = SplitWithQuote(cmd, " ");		// コマンドライン引数(分解済)
var opts = [];								// オプション(分解済)
var procedDocs = [];						// 処理したドキュメント一覧
//var $isAutoCreateNewFile = false;			// 存在しないファイルを自動作成するか
var fso = new ActiveXObject("Scripting.FileSystemObject");
var skip = 0;
for (var i=1; i<args.length; i++) {
	// 標準オプションで追加引数をスキップするための処理(/cl など)
	if (skip > 0) {
		skip--;
		continue;
	}

	//-------------------------------------------------
	// 逐次オプション処理
	// 処理されなかった場合はオプションキューに詰める
	if (args[i].startsWith("/")) {
		var opt = SplitWithQuote(args[i], ":");

		var isProced = false;
		for (var j=0; j<seqOptFunc.length; j++) {
			isProced = seqOptFunc[j](opt);
			if (isProced) { break; }
		}
		if (!isProced) {
			opts[opts.length] = opt;
		}

		// スキップ指定戻り値の処理(標準オプション用の特別処理)
		if (typeof isProced == "number") {
			skip = isProced - 1;
			continue;
		}
	} else {
		//-------------------------------------------------
		// ファイル名処理開始
		args[i] = args[i].dequote();

		//// ファイルが開かれていない場合は先回りして開く
		//var doc = SearchDocument(args[i]);
		//if (!doc) {
		//	var path = fso.GetAbsolutePathName(args[i]);
		//	if (fso.FileExists(path)) {
		//		Editor.NewFile();
		//		Editor.OpenFile(path);
		//		doc = Editor.Documents.Item(Editor.Documents.Count-1);
		//		UpdateDocumentTable();
		//	} else if ($isAutoCreateNewFile) {
		//		// 自動作成
		//		Editor.NewFile();
		//		doc = Editor.Documents.Item(Editor.Documents.Count-1);
		//		doc.Save(path);
		//		UpdateDocumentTable();
		//	}
		//} else {
		//	doc.Activate();
		//}

		// ファイル名から対応するドキュメントを取得
		// 絶対パスを正規化して判定
		if (fso.FileExists(fso.GetAbsolutePathName(args[i]))) {
			var doc = SearchDocument(args[i]);
			if (doc) {
				procedDocs[procedDocs.length] = doc;
			}

			// オプション処理
			for (var j=0; j<opts.length; j++) {
				for (var k=0; k<docOptFunc.length; k++) {
					if (docOptFunc[k](opts[j], doc)) {
						// true の場合はオプションをキューから削除
						opts.splice(j, 1);
						j--;
						break;
					}
				}
			}
		}
	}
}

// 完了時実行オプションを実行
for (var i=0; i<opts.length; i++) {
	for (var j=0; j<lastOptFunc.length; j++) {
		lastOptFunc[j](opts[i], procedDocs);
	}
}
